import numpy as np
import os
import sys
import cv2
import torch.optim as optim
from PIL import Image
sys.path.insert(0,'../')
from model import *
from utils import *
from region_to_bbox import *
from visualization import *

cf_num_channels = 32
vgg_num_channels = 512
output_sigma_factor = 0.1
motion_sigma_factor = 0.6
cell_sz = 4.0

online_lr_base = 0.000007
online_w_decay = 0
n_init_updates = 1
n_online_updates = 2
update_interval = 5
n_update_batch = 5
max_db_size = n_online_updates*n_update_batch
#cf1 parameters
init_cf1_num = 100
cf_lr = 7e-5
cf2_lr = 7e-6
use_gpu = 1

feat_extractor_path = '../models/imagenet-vgg-verydeep-16.mat'
tracker_path = '../models/meta_init_vot_ilsvrc_32.pthiter10000'
tracker = 'init%02d-lr%.6f'%(n_init_updates, online_lr_base)

score_thres = 0.0

np.random.seed(4)
torch.manual_seed(5)
torch.cuda.manual_seed(6)

class meta_filter(nn.Module):
    def __init__(self, img_first, bbox_first, scale):
        super(meta_filter, self).__init__()
        self.feature_net = FeatureExtractor(feat_extractor_path)
        if use_gpu:
            self.feature_net = self.feature_net.cuda()

        ## MetaCrest init ##
        state = torch.load(tracker_path)
        self.meta_init = state['meta_init']
        self.meta_alpha = state['meta_alpha']
        self.criterion = L2normLoss2()
        if use_gpu:
            self.criterion = self.criterion.cuda()

        # Generate sequence config
        target_bbox1 = np.array(bbox_first)
        max_object_sz = 127.

        ## resize target object size - computational issue ##
        # scale = 1
        # if target_bbox1[2:].prod() > max_object_sz ** 2:
        #     scale = max_object_sz / (target_bbox1[2:].max())
        self.scale = scale
            # print('processing seq:%s, resized'%seq)
            # else:
            # print('processing seq:%s'%seq)
        # load first image1
        image1 = Image.open(img_first).convert('RGB')
        image_sz_resized = (int(image1.size[0] * scale), int(image1.size[1] * scale))
        self.image_sz_resized = image_sz_resized
        image1_resized = image1.resize(image_sz_resized, Image.BILINEAR)
        # scale target bbox
        resized_target_bbox1 = np.round(target_bbox1 * scale)
        target_sz1 = resized_target_bbox1[2:]
        self.target_sz = target_sz1
        target_center1 = resized_target_bbox1[:2] + np.floor(target_sz1 / 2)
        # get cosine window(square shape)
        window_sz = get_search_size(target_sz1)
        self.window_sz = window_sz
        window_cell_sz = np.ceil(window_sz / cell_sz)
        window_cell_sz = window_cell_sz - (window_cell_sz % 2) + 1
        self.window_cell_sz = window_cell_sz
        cos_window = np.outer(np.hanning(window_cell_sz), np.hanning(window_cell_sz))
        cos_window = cos_window[None, None, :, :].repeat(vgg_num_channels, axis=1)
        cos_window = Variable(torch.from_numpy(cos_window).float(), requires_grad=False)
        if use_gpu:
            cos_window = cos_window.cuda()
        self.cos_window = cos_window
        # get search patch and extract the feature
        patch = get_search_patch(image1_resized, target_center1, window_sz)
        patch = Variable(torch.from_numpy(patch[None, :]))
        if use_gpu:
            patch = patch.cuda()
        feat = self.feature_net.forward(patch)
        # resizing features as same size as cos_window
        feat = self.resize_feat_cos_window(feat, cos_window)
        self.feat = feat
        feat_sz = feat.size(3)

        ## get correlation filter label(gaussian shape) ##
        target_cell_sz1 = np.ceil(target_sz1 / cell_sz)
        target_cell_sz1 = target_cell_sz1 - (target_cell_sz1 % 2) + 1
        rw = int(np.ceil(target_cell_sz1[0] / 2))  # padding for CF
        rh = int(np.ceil(target_cell_sz1[1] / 2))  # padding for CF
        fw = int(rw * 2 + 1)
        fh = int(rh * 2 + 1)
        self.rw = rw
        self.rh = rh
        self.target_cell_sz = target_cell_sz1

        output_sigma = target_cell_sz1 * output_sigma_factor
        label1_np = gaussian_shaped_labels_m(output_sigma, feat_sz, target_sz1)
        label1 = Variable(torch.from_numpy(label1_np).float())
        label1 = label1.view(1, 1, label1.size(0), label1.size(1))
        if use_gpu:
            label1 = label1.cuda()
        self.label_map = label1
        self.tracker_net = self.train_init0()

    def train_init0(self):
        meta_init = self.meta_init
        meta_alpha = self.meta_alpha
        target_cell_sz = self.target_cell_sz
        loss_fn = self.criterion
        feat = self.feat
        label_map = self.label_map
        rw = int(np.ceil(target_cell_sz[0] / 2))
        rh = int(np.ceil(target_cell_sz[1] / 2))
        fw = int(rw * 2 + 1)
        fh = int(rh * 2 + 1)

        # bilinear sampling for resizing correlation filters
        cf_init_weight = OrderedDict()
        for k, p in meta_init.items():
            if 'cf.weight' in k:
                cf_init_weight[k] = bilinear_upsample(p, (fh, fw))
            else:
                cf_init_weight[k] = 1 * p
        cf_init_alpha = OrderedDict()
        for k, p in meta_alpha.items():
            if 'cf.weight' in k:
                cf_init_alpha[k] = bilinear_upsample(p, (fh, fw))
            else:
                cf_init_alpha[k] = 1 * p

        # the first iteration
        temp = conv2d(feat, cf_init_weight['conv1.weight'], cf_init_weight['conv1.bias'],
                      stride=1, padding=0)
        response = conv2d(temp, cf_init_weight['cf.weight'], cf_init_weight['cf.bias'],
                          stride=1, padding=(rh, rw))

        loss = loss_fn(response, label_map, None)
        # print('Init loss: %.4f' % (loss.data[0]))
        grads = torch.autograd.grad(loss, cf_init_weight.values(), create_graph=True)
        cf_weight = OrderedDict((name, param - torch.mul(alpha, grad)) for
                                ((name, param), (_, alpha), grad) in
                                zip(cf_init_weight.items(), cf_init_alpha.items(), grads))
        # subsequent iterations
        for i in range(1):
            temp = conv2d(feat, cf_weight['conv1.weight'], cf_weight['conv1.bias'],
                          stride=1, padding=0)
            response = conv2d(temp, cf_weight['cf.weight'], cf_weight['cf.bias'],
                              stride=1, padding=(rh, rw))
            loss = loss_fn(response, label_map, None)
            grads = torch.autograd.grad(loss, cf_weight.values(), create_graph=True)
            a_grads = torch.autograd.grad(loss, cf_init_alpha.values(), create_graph=True)
            cf_alpha = OrderedDict((name, param - 1e-6 * grad)
                                   for ((name, param), grad) in
                                   zip(cf_init_alpha.items(), a_grads))
            cf_weight = OrderedDict((name, param - torch.mul(alpha, grad))
                                    for ((name, param), (_, alpha), grad) in
                                    zip(cf_weight.items(), cf_alpha.items(), grads))

            # print('loss: %.4f' % (loss.data[0]))

        tracker_net = CFConv1NetOnline(fh, fw, rh, rw, vgg_num_channels, cf_num_channels)
        tracker_net.copy_meta_weights(cf_init_weight)
        self.alpha = cf_alpha
        self.init = cf_weight
        return tracker_net

    def train_mid(self, image_train_all, bbox_train_all, mask):
        loss_fn = self.criterion
        label_map = self.label_map
        # bilinear sampling for resizing correlation filters
        cf_weight = self.init
        cf_alpha = self.alpha
        len_ = len(image_train_all)
        # feat_all = []
        loss_ = 0
        for i in (0,len_-1):
            image_train = image_train_all[i]
            bbox_train = bbox_train_all[i]
            target_bbox1 = np.array(bbox_train)
            # load first image1
            # image1 = Image.open(image_train).convert('RGB')
            # # image_sz_resized = (int(image1.size[0] * self.scale), int(image1.size[1] * self.scale))
            # image1_resized = image1.resize(self.image_sz_resized, Image.BILINEAR)
            # scale target bbox
            resized_target_bbox1 = target_bbox1#np.round(target_bbox1 * self.scale)
            target_sz1 = resized_target_bbox1[2:]
            target_center1 = resized_target_bbox1[:2] + np.floor(target_sz1 / 2)
            patch = get_search_patch(image_train, target_center1, self.window_sz)
            patch = Variable(torch.from_numpy(patch[None, :]))
            if use_gpu:
                patch = patch.cuda()
            feat = self.feature_net.forward(patch)
            # resizing features as same size as cos_window
            feat = self.resize_feat_cos_window(feat, self.cos_window)

        # the first iteration
        # temp = conv2d(feat, cf_init_weight['conv1.weight'], cf_init_weight['conv1.bias'],
        #               stride=1, padding=0)
        # response = conv2d(temp, cf_init_weight['cf.weight'], cf_init_weight['cf.bias'],
        #                   stride=1, padding=(self.rh, self.rw))
        #
        # loss = loss_fn(response, label_map, None)
        # # print('Init loss: %.4f' % (loss.data[0]))
        # grads = torch.autograd.grad(loss, cf_init_weight.values(), create_graph=True)
        # cf_weight = OrderedDict((name, param - torch.mul(alpha, grad)) for
        #                         ((name, param), (_, alpha), grad) in
        #                         zip(cf_init_weight.items(), cf_init_alpha.items(), grads))
            temp = conv2d(feat, cf_weight['conv1.weight'], cf_weight['conv1.bias'],
                          stride=1, padding=0)
            response = conv2d(temp, cf_weight['cf.weight'], cf_weight['cf.bias'],
                              stride=1, padding=(self.rh, self.rw))
            loss = loss_fn(response, label_map, mask)
            loss_ = loss_+loss
            # a_grads = torch.autograd.grad(loss, cf_init_alpha.values(), create_graph=True)
            # cf_alpha = OrderedDict((name, param - 1e-6 * grad)
            #                        for ((name, param), grad) in
            #                        zip(cf_init_alpha.items(), a_grads))
        grads = torch.autograd.grad(loss, cf_weight.values(), create_graph=True)
        cf_weight = OrderedDict((name, param - torch.mul(alpha, grad))
                                for ((name, param), (_, alpha), grad) in
                                zip(cf_weight.items(), cf_alpha.items(), grads))

        # tracker_net = CFConv1NetOnline(fh, fw, rh, rw, vgg_num_channels, cf_num_channels)
        self.tracker_net.copy_meta_weights(cf_weight)
        # return tracker_net


    # resizing features as same size as cos_window
    def resize_feat_cos_window(self, feat, cos_window):
        num_channels = feat.size(1)
        cos_window_sz = cos_window.size(3)
        feat_data = feat.data.clone().cpu().numpy()
        # t1 = time.time()
        feat = cv2.resize(feat_data[0].transpose(1, 2, 0).astype('float32'), dsize=(cos_window_sz, cos_window_sz))
        feat = feat.transpose(2, 0, 1)
        feat = Variable(torch.from_numpy(feat).float(), requires_grad=False)
        if use_gpu:
            feat = feat.cuda()
        # t2 = time.time() - t1
        # print('resize--%.4f' % t2)
        feat = feat * cos_window
        return feat

    def set_optimizer(self,model, lr_base, w_decay):
        param_list = []
        for k, p in model.named_parameters():
            param_list.append({'params': [p], 'lr': lr_base, 'weight_decay': w_decay})
        optimizer = optim.Adam(param_list)
        return optimizer


    def set_max_object_sz(self, seq_name):
        if seq_name == 'Jump':
            return 70
        elif seq_name == 'Skater2':
            return 70
        elif seq_name == 'Girl2':
            return 70
        else:
            return 100

    def forward(self, img_now, target_center):
        self.tracker_net.eval()
        window_sz = self.window_sz
        window_cell_sz = self.window_cell_sz
        # image = Image.open(img_now).convert('RGB')
        # # image_sz_resized = (int(image.size[0] * scale), int(image.size[1] * scale))
        # image_resized = image.resize(self.image_sz_resized, Image.BILINEAR)
        patch = get_search_patch(img_now, target_center, self.window_sz)
        patch = Variable(torch.from_numpy(patch[None, :]))
        if use_gpu:
            patch = patch.cuda()

        feat = self.feature_net.forward(patch)
        # t_feat1 = time.time()
        feat = self.resize_feat_cos_window(feat, self.cos_window)
        # t_feat2 = time.time() - t_feat1
        # print('time_extract_feat--%.4f' % t_feat2)

        # t_res1 = time.time()
        response = self.tracker_net.forward(feat)
        # t_res2 = time.time()-t_res1
        # print('time_get_response--%.4f' %t_res2)
        score = response.max().data[0]
        response = response - response.min()
        response = response / response.max()
        if score<0.15:
            return target_center, response

        motion_sigma = self.target_cell_sz * motion_sigma_factor
        motion_map = gaussian_shaped_labels_m(motion_sigma, window_cell_sz, self.target_sz)
        motion_map = Variable(torch.from_numpy(motion_map).float())
        motion_map = motion_map.view(1, 1, motion_map.size(0), motion_map.size(1))
        if use_gpu:
            motion_map = motion_map.cuda()
        response2 = response
        response = response * motion_map
        response_np = response.data[0][0].cpu().numpy()
        delta = np.argwhere(response_np.max() == response_np)[0] + 1  # zero-index -> original-index
        delta = delta - np.ceil(window_cell_sz / 2)
        # delta = delta - (window_cell_sz / 2)
        target_center[0] = target_center[0] + delta[1] * cell_sz
        target_center[1] = target_center[1] + delta[0] * cell_sz
        target_center1 = valid_pos(target_center, self.image_sz_resized)
        response2 = response2-response2.min()
        response2 = response2/response2.max()
        return target_center1, response2


