#!/usr/bin/python

import vot
import sys
sys.path.append('/home/lpx/anaconda3/lib/python3.6/site-packages/')
import torch
from torch.autograd import Variable
import torch.utils.data as data
import torch.optim as optim
import numpy as np
import os
import time
import argparse
import json
from PIL import Image
import cv2


sys.path.insert(0, '/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/')
from model_siam import *
from utils import *
from region_to_bbox import *
from visualization import *

sys.path.insert(0, '/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/mdnet_model')
from mdnet_model import *

# sys.path.insert(0, '/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/yolov3')
# from yolov3 import *

module_path = os.path.abspath(os.path.join('/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/ssd'))
if module_path not in sys.path:
    sys.path.append(module_path)
if torch.cuda.is_available():
    torch.set_default_tensor_type('torch.cuda.FloatTensor')
sys.path.insert(0, '/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/ssd')
from ssd import build_ssd

cf_num_channels = 32
vgg_num_channels = 512
output_sigma_factor = 0.1
motion_sigma_factor = 0.6
cell_sz = 4.0
exemplar_sz = 127
search_sz = 255
score_sz2 = 264
score_sz = 33
context = 0.5
feat_extractor_path = '/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/models/baseline-conv5_e55.mat'

np.random.seed(4)
torch.manual_seed(5)
torch.cuda.manual_seed(6)


def ssd_detection(image_file, ssd_net):
    image = cv2.imread(image_file)
    rgb_image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    x = cv2.resize(image, (300, 300)).astype(np.float32)
    x -= (104.0, 117.0, 123.0)
    x = x.astype(np.float32)
    x = x[:, :, ::-1].copy()
    x = torch.from_numpy(x).permute(2, 0, 1)
    xx = Variable(x.unsqueeze(0))  # wrap tensor in Variable
    if torch.cuda.is_available():
        xx = xx.cuda()
    y = ssd_net(xx)
    detections = y.data
    scale = torch.Tensor(rgb_image.shape[1::-1]).repeat(2)
    detection_bbox = []
    for i in range(detections.size(1)):
        j = 0
        while detections[0, i, j, 0] > 0.:
            score = detections[0, i, j, 0]
            pt = (detections[0, i, j, 1:] * scale).cpu().numpy()
            detection_bbox_ = np.array([pt[0], pt[1], pt[2] - pt[0] + 1, pt[3] - pt[1] + 1, score], dtype=np.float32)
            detection_bbox.append(detection_bbox_)
            j+=1

    return detection_bbox

class SYTTracker(object):

    def __init__(self, image_file, region):

        target_bbox1 = np.array(region)

        image1 = Image.open(image_file).convert('RGB')
        self.image_sz_resized = (int(image1.size[0]), int(image1.size[1]))
        image1_resized = image1.resize(self.image_sz_resized, Image.BILINEAR)

        # scale target bbox
        resized_target_bbox1 = np.round(target_bbox1)
        target_sz1 = resized_target_bbox1[2:]
        target_center1 = resized_target_bbox1[:2] + np.floor(target_sz1 / 2)
        # get cosine window(square shape)
        context_ = context * (target_sz1[0] + target_sz1[1])
        target_sz1_ = np.sqrt((target_sz1[0] + context_) * (target_sz1[1] + context_))
        self.window_sz_ = float(search_sz) / exemplar_sz * target_sz1_
        self.target_sz_ = target_sz1_

        '''init mdnet model'''
        self.mdnet_model_ = mdnet_model(image1_resized, resized_target_bbox1)

        cos_window = np.outer(np.hanning(score_sz2), np.hanning(score_sz2))
        cos_window = cos_window[None, None, :, :].repeat(1, axis=1)
        cos_window = Variable(torch.from_numpy(cos_window).float(), requires_grad=False)
        self.cos_window = cos_window.cuda()

        cos_window_p = np.outer(np.hanning(12), np.hanning(12))
        cos_window_p = cos_window_p[None, None, :, :].repeat(1, axis=1)
        cos_window_p = Variable(torch.from_numpy(cos_window_p).float(), requires_grad=False)
        cos_window_p = cos_window_p.cuda()

        '''initialize the siamese fc network'''
        self.feature_net = FeatureExtractor_siam(feat_extractor_path)
        self.feature_net = self.feature_net.cuda()
        self.feature_net.eval()

        '''utilize the first frame as template'''
        patch0 = get_search_patch_siam(image1_resized, target_center1, target_sz1_)
        patch0 = Variable(torch.from_numpy(patch0[None, :]))
        patch0 = bilinear_upsample(patch0, [exemplar_sz, exemplar_sz])
        patch0 = patch0.cuda()
        feat0 = self.feature_net.forward(patch0)
        self.template = feat0

        # '''initialize yolov3 model'''
        # self.yolov3_net_ = yolov3_net()  # initialize yolov3
        self.ssd_net = build_ssd('test', 300, 21)  # initialize SSD
        self.ssd_net.load_weights('/home/lpx/Desktop/code-vot2018/VOT2018/vot-toolkit-master/SYT_W/SYT/ssd/weights/ssd300_mAP_77.43_v2.pth')

        '''parameters'''
        self.target_cell_sz1 = np.ceil(target_sz1_ / cell_sz)
        self.target_center = target_center1
        self.target_sz = target_sz1
        self.resized_target_bbox = resized_target_bbox1.copy()
        self.window_sz_min = self.window_sz_ * 0.2
        self.window_sz_max = self.window_sz_ * 5.
        self.scales = [1, 0.962, 1.04]
        self.frame_num = 0
        self.resized_target_bbox_keep = resized_target_bbox1.copy()
        self.img_train_all = []
        self.label_train_all = []
        self.frame_train_all = []
        self.img_train_all.append(image1_resized)
        self.label_train_all.append(self.resized_target_bbox)
        self.frame_train_all.append(self.frame_num)
        self.train_frame = 0
        self.confidence = 0

    def track(self, image_file):
        # self.resized_target_bbox_keep = self.resized_target_bbox1.copy()
        self.frame_num += 1
        # confidence = 0
        '''siamenseFC tracking'''
        image = Image.open(image_file).convert('RGB')
        image_resized = image.resize(self.image_sz_resized, Image.BILINEAR)
        patch_ = get_search_patch_scale(image_resized, self.target_center, self.window_sz_)
        feat = self.feature_net.forward(patch_)
        response_ = self.feature_net.siam(feat, self.template)
        response_[1, :, :] = response_[1, :, :] * 0.97
        response_[2, :, :] = response_[2, :, :] * 0.97
        response_d = response_.data.cpu().numpy()
        new_scale_id = np.argmax(np.amax(response_d, axis=(1, 2)))
        window_sz_ = self.window_sz_ * self.scales[new_scale_id]
        if window_sz_ < self.window_sz_min:
            # target_sz_ = target_sz_min
            window_sz_ = self.window_sz_min
            new_scale_id = 0
        if window_sz_ > self.window_sz_max:
            # target_sz_ = target_sz_max
            window_sz_ = self.window_sz_max
            new_scale_id = 0

        response = response_[new_scale_id][None, None, :, :]
        response = bilinear_upsample(response, [score_sz2, score_sz2])
        score = response.max().data[0]
        if self.frame_num == 1:
            score_init = score
        response.data = response.data - response.data.min()
        response.data = response.data / (response.data.max()+0.00000001)
        response.data = (1 - 0.25) * response.data + 0.25 * self.cos_window.data

        response_np = response.data[0][0].cpu().numpy()
        delta = np.argwhere(response_np.max() == response_np)[0] + 1  # zero-index -> original-index
        delta = delta - np.ceil((score_sz2) / 2.)
        delta_xcrop = delta * cell_sz * score_sz / score_sz2
        delta_in_frame = delta_xcrop * int(window_sz_) / search_sz
        self.target_center[0] = self.target_center[0] + delta_in_frame[1]
        self.target_center[1] = self.target_center[1] + delta_in_frame[0]
        self.target_center = valid_pos(self.target_center, self.image_sz_resized)
        self.target_sz = 0.41 * self.target_sz + 0.59 * self.target_sz * self.scales[new_scale_id]
        self.resized_target_bbox_keep = np.zeros(4)
        self.resized_target_bbox_keep[:2] = self.target_center - self.target_sz / 2
        self.resized_target_bbox_keep[2:] = self.target_sz
        context_ = context * (self.target_sz[0] + self.target_sz[1])
        target_sz_ = np.sqrt((self.target_sz[0] + context_) * (self.target_sz[1] + context_))

        '''evaluate the tracking score'''
        mdnet_score = self.mdnet_model_.forward(image_resized, self.resized_target_bbox_keep)
        if self.frame_num==1:
            self.init_mdnet_score = mdnet_score[0,1]

        '''tracking or detection'''
        if mdnet_score[0, 1] < 0.:
            if self.train_frame not in self.frame_train_all[1:] and len(self.frame_train_all)>0:
                self.mdnet_model_.re_train(self.img_train_all, self.label_train_all, self.frame_num)
                self.train_frame = self.frame_train_all[-1]
            # prediction_bbox = self.yolov3_net_.forward(image_file)
            # if len(prediction_bbox):
            #     detection_bbox = prediction_bbox.cpu().numpy()
            #     detection_bbox[:, 3:5] = detection_bbox[:, 3:5] - detection_bbox[:, 1:3]
            # else:
            #     detection_bbox = []
            detection_bbox = ssd_detection(image_file, self.ssd_net)
            # '''get the top10 bounding box and use mdnet to choose the best one'''
            # top10_bbox = np.zeros([5, 5], np.float32)
            bbox_all = []
            score_all = []
            confidence_all = []
            iii = 0
            for jj in range(len(detection_bbox)):
                ref_ratio = self.target_sz[0]/self.target_sz[1]
                now_ratio = detection_bbox[jj][2]/detection_bbox[jj][3]
                b_ratio = now_ratio/ref_ratio
                # size_ratio = (detection_bbox[jj][2:4]/target_sz).max()
                if (b_ratio>0.5) and (b_ratio<2.):# and (size_ratio<2.):
                    bbox_all.append(detection_bbox[jj][0:4])
                    mdnet_score = self.mdnet_model_.forward(image_resized, detection_bbox[jj][0:4])
                    score_all.append(mdnet_score[0, 1])
                    confidence_all.append(detection_bbox[jj][4])
            # for jj in range(len(detection_bbox)):
            #     mdnet_score = self.mdnet_model_.forward(image_resized, detection_bbox[jj][1:5])
            #     if mdnet_score[0, 1] > 0.:
            #         bbox_all.append(detection_bbox[jj][1:5])
            #         score_all.append(mdnet_score[0,1])
            #         confidence_all.append(detection_bbox[jj][5])

            if not len(score_all)==0:
                score_all = np.stack(score_all, 0)
                idx_ = np.argmax(score_all)
                if score_all[idx_] > 0.:
                    self.confidence = confidence_all[idx_]
                    self.resized_target_bbox_keep = bbox_all[idx_]
                    self.resized_target_bbox = bbox_all[idx_]
                    self.target_sz = self.resized_target_bbox_keep[2:]
                    self.target_center = self.resized_target_bbox_keep[:2] + self.target_sz / 2
        else:
            if mdnet_score[0, 1] > self.init_mdnet_score/4.:
                # print('tracking')
                self.confidence = 1.
                self.resized_target_bbox = np.copy(self.resized_target_bbox_keep)
                self.target_sz = self.resized_target_bbox_keep[2:]
                self.target_center = self.resized_target_bbox_keep[:2] + self.target_sz / 2
                self.mdnet_model_.sample_generator.set_trans_f(0.6)
            else:
                self.confidence = 0.7 * self.confidence + 0


        '''save training samples'''
        if mdnet_score[0, 1] > self.init_mdnet_score / 2.:
            patchn = get_search_patch_siam(image_resized, self.target_center, self.target_sz_)
            patchn = Variable(torch.from_numpy(patchn[None, :]))
            patchn = bilinear_upsample(patchn, [exemplar_sz, exemplar_sz])
            patchn = patchn.cuda()
            featn = self.feature_net.forward(patchn)
            # template.data = (1 - 0.01) * template.data + 0.01 * featn.data
            image_train = image_resized
            bbox_train = self.resized_target_bbox
            if len(self.img_train_all) < 5:
                self.img_train_all.append(image_train)
                self.label_train_all.append(bbox_train)
                self.frame_train_all.append(self.frame_num)
            elif (self.frame_num - self.frame_train_all[-1]) > 5:
                del self.img_train_all[1]
                del self.label_train_all[1]
                del self.frame_train_all[1]
                self.img_train_all.append(image_train)
                self.label_train_all.append(bbox_train)
                self.frame_train_all.append(self.frame_num)

        region = np.round(self.resized_target_bbox)
        context_ = context * (self.target_sz[0] + self.target_sz[1])
        self.target_sz_ = np.sqrt((self.target_sz[0] + context_) * (self.target_sz[1] + context_))
        self.window_sz_ = float(search_sz) / exemplar_sz * self.target_sz_

        return vot.Rectangle(region[0], region[1], region[2], region[3]), self.confidence

handle = vot.VOT("rectangle")
selection = handle.region()

imagefile = handle.frame()
if not imagefile:
    sys.exit(0)

# image = cv2.imread(imagefile, cv2.IMREAD_GRAYSCALE)
tracker = SYTTracker(imagefile, selection)
while True:
    imagefile = handle.frame()
    if not imagefile:
        break
    # image = cv2.imread(imagefile, cv2.IMREAD_GRAYSCALE)
    region, confidence = tracker.track(imagefile)
    handle.report(region, confidence)
